from mag_tools.enums.base_enum import BaseEnum


class UsbType(BaseEnum):
    STORAGE = ("Storage", "存储设备")
    HUB = ("Hub", "集线器")
    COMPOSITE = ("Composite", "复合设备")
    KEY = ("Key", "加密狗")
    OTHER = ("Other", "其他设备")



import socket
from dataclasses import dataclass, field
from typing import  Optional

from mag_tools.bean.base_data import BaseData
from mag_tools.bean.sys.memory_module import MemoryModule
from mag_tools.bean.sys.cpu import Cpu
from mag_tools.bean.sys.disk import Disk
from mag_tools.bean.sys.memory import Memory
from mag_tools.bean.sys.mother_board import Motherboard
from mag_tools.enums.computer_type import ComputerType
from mag_tools.utils.security.digest import Digest


@dataclass
class Computer(BaseData):
    """
    计算机类
    """
    type: ComputerType = field(default=None, metadata={"description": "计算机类型"})
    id: Optional[str] = field(default=None, metadata={"description": "计算机标识"})
    name: Optional[str] = field(default=None, metadata={"description": "计算机名"})
    description: Optional[str] = field(default=None, metadata={"description": "描述"})
    cpu: Optional[Cpu] = field(default=None, metadata={"description": "CPU信息"})
    memory: Optional[Memory] = field(default=None, metadata={"description": "内存信息"})
    memory_modules: list[MemoryModule] = field(default_factory=list, metadata={"description": "内存条信息"})
    disks: list[Disk] = field(default_factory=list, metadata={"description": "磁盘信息"})
    mother_board: Optional[Motherboard] = field(default_factory=list, metadata={"description": "主板信息"})

    @classmethod
    def get_info(cls):
        """
        获取当前系统的CPU、内存和磁盘信息，并返回一个Computer实例
        """
        pc = cls(type=ComputerType.DESKTOP,
                 name=socket.gethostname(),
                 cpu=Cpu.get_info(),
                 memory=Memory.get_info(),
                 memory_modules=MemoryModule.get_info(),
                 disks=Disk.get_info(),
                 mother_board=Motherboard.get_info())

        pc.__set_id()
        return pc

    def __set_id(self):
        modules_uuid = ''.join([module.hash for module in self.memory_modules])
        disks_uuid = ''.join([disk.hash for disk in self.disks])
        self.id = Digest.md5(f'{self.name}{self.type}{self.cpu.hash}{self.memory.hash}{modules_uuid}{disks_uuid}{self.mother_board}')

        self.cpu.computer_id = self.id
        self.mother_board.computer_id = self.id
        self.memory.computer_id = self.id

        for disk in self.disks:
            disk.computer_id = self.id
        for module in self.memory_modules:
            module.computer_id = self.id


if __name__ == "__main__":
    pc_ = Computer.get_info()
    print(pc_)



import re

class NumberValidator:
    """
    数据格式检测类。

    @description: 提供数据格式检查。
    @version: v1.3
    @date: 2016/7/10
    """
    @staticmethod
    def is_scientific_number(string: str) -> bool:
        """
        判定一个字符串是否为科学记数法

        :param string: 科学记数法表示的字符串
        :return: 是否为科学记数法
        """
        if not string:
            return False
        # 正则表达式匹配科学记数法
        regex = r"^[+-]?\d+(\.\d*)?([Ee][+-]?\d+)?$"
        return bool(re.match(regex, string))

    @staticmethod
    def is_numeric(string: str) -> bool:
        """
        判定一个字符串是否为数值（包括浮点数和整数）

        :param string: 数值字符串
        :return: 是否为数值
        """
        try:
            if string and len(string) > 0:
                float(string)
                return True
        except ValueError:
            pass
        return False



import unittest
from types import NoneType
from typing import Optional

from enums.client_type import ClientType
from utils.common.class_utils import ClassUtils


class TestClassUtils(unittest.TestCase):
    def test_get_origin_type(self):
        print(ClassUtils.get_origin_type(Optional[str]))
        print(ClassUtils.get_origin_type(Optional[int]))
        print(ClassUtils.get_origin_type(int))
        print(ClassUtils.get_origin_type(NoneType))
        print(ClassUtils.get_origin_type(ClientType))
        print(ClassUtils.get_origin_type(tuple[int]))
        print(ClassUtils.get_origin_type(list[list[int]]))


from dataclasses import dataclass, field
from types import NoneType
from typing import Any, Optional, Type, get_origin, get_args, Union, get_type_hints


class ClassUtils:
    @staticmethod
    def get_origin_type(input_type: Type) -> Type | None:
        """
        获取原始类型
            判断输入是否为 Optional 类型，如果是，则返回其内部类型。

        :param input_type: 待检查的类型
        :return: Optional 内部的类型（如果是 Optional），否则返回原始类型或 None
        """
        origin_type = get_origin(input_type)
        if origin_type is Union:  # 判断是否为 Union 类型
            args = get_args(input_type)  # 提取 Union 内的类型参数
            # 判断是否恰好包含两个类型，并且其中一个是 NoneType
            if len(args) == 2 and NoneType in args:
                return next(arg for arg in args if arg is not NoneType)
        # 如果不是 Optional 类型，直接返回输入类型
        return origin_type if origin_type else input_type

    @staticmethod
    def isinstance(bean: Any, cls_types: [Type, ...]) -> bool:
        return any(isinstance(bean, cls_type) for cls_type in cls_types)

    @staticmethod
    def get_field_type_of_class(clazz: type, field_name: str) -> Type | None:
        """
        获取指定字段名的类型。
        :param clazz: 要检查的类
        :param field_name: 字段名
        :return: 字段的类型，如果字段不存在则返回 None
        """
        field_type = get_type_hints(clazz)
        return field_type.get(field_name, None)

    @staticmethod
    def get_field_type_of_bean(bean: Any, field_name: str) -> Optional[Type]:
        """
        根据对象和字段名获取字段的类型

        :param bean: 对象
        :param field_name: 字段名
        :return: 字段的类型
        """
        # 获取对象的类型注解
        annotations = getattr(bean, '__annotations__', {})

        # 如果字段在类型注解中，返回类型注解
        if field_name in annotations:
            return annotations[field_name]

        # 否则，尝试获取实例变量的类型
        return type(getattr(bean, field_name, None))


# 示例使用
@dataclass
class MyClass:
    name: str = field()
    age: int = field()

if __name__ == '__main__':
    obj = MyClass("Alice", 30)
    print(ClassUtils.get_field_type_of_bean(obj, 'name'))  # 输出: <class 'str'>
    print(ClassUtils.get_field_type_of_bean(obj, 'age'))  # 输出: <class 'int'>
    print(ClassUtils.get_field_type_of_class(MyClass, 'name'))  # 输出: <class 'str'>



from dataclasses import dataclass, field

@dataclass
class DbPage:
    """
    分页类

    @description: 提供基础的分页操作
    @version: 2.5
    @date: 2015
    """
    index: int = field(default=1, metadata={"description": "页面索引，从1开始"})
    size: int = field(default=10, metadata={"description": "单页最大纪录数，缺省为10"})
    order_column: str = field(default=None, metadata={"description": "用于排序的列名"})
    ascend: bool = field(default=True, metadata={"description": "是否升序"})
    __total_page: int = field(default=-1, metadata={"description": "总页数"})
    total_num: int = field(default=-1, metadata={"description": "总纪录数"})
    total_sql: str = field(default=None, metadata={"description": "计算总纪录数的SQL语句,缺省为空"})
    current_page: int = field(default=1, metadata={"description": "当前请求的页数"})
    uuid: str = field(default=None, metadata={"description": "分页查询id，上一次查询返回的uuid,缺省为空"})

    @property
    def first_row(self) -> int:
        """
        获取首记录行号

        :return: 首记录行号
        """
        return (self.index - 1) * self.size

    @property
    def total_page(self) -> int:
        """
        取得总页数，-1表示错误

        :return: 总页数
        """
        if self.size < 1 or self.total_num < 0:
            return -1

        quotient = self.total_num // self.size
        remainder = self.total_num % self.size
        if remainder == 0:
            self.__total_page = quotient
        else:
            self.__total_page = quotient + 1

        return self.__total_page

    @property
    def offset(self):
        return (self.index - 1) * self.size

    def get_sql(self):
        return f"LIMIT {self.size} OFFSET {self.offset}"

    def __str__(self) -> str:
        return f"DbPage{{totalNum={self.total_num},totalSql={self.total_sql},size={self.size},totalPage={self.total_page},currentPage={self.current_page}}}"



from dataclasses import MISSING, dataclass, fields
from typing import Any,   Optional, Union

import random

from mag_tools.format.text_formatter import TextFormatter
from mag_tools.enums.justify_type import JustifyType


@dataclass
class BaseData:
    """
    Bean的基类
    """
    _formatter = TextFormatter(
        number_per_line=None,
        justify_type=JustifyType.LEFT,
        at_header='',
        at_end='',
        decimal_places=6,
        decimal_places_of_zero=1,
        pad_length=None,
        pad_char=' ',
        scientific=False,
        none_default='NA'
    )

    def __repr__(self):
        field_dict = {k: v for k, v in self.__dict__.items() if not k.startswith('_')}
        field_str = ', '.join(f"{k}={v}" for k, v in field_dict.items())
        return f"{self.__class__.__name__}({field_str})"

    def __str__(self):
        excluded_attributes = {}
        attributes = [
            f"{attr}={repr(getattr(self, attr))}"
            for attr in vars(self)
            if getattr(self, attr) is not None and attr not in excluded_attributes
        ]
        return f"{self.__class__.__name__}({', '.join(attributes)})"

    def __getitem__(self, item):
        if hasattr(self, item):
            return getattr(self, item)
        raise KeyError(f"{item} not found in {self.__class__.__name__}")

    @classmethod
    def get_default_value(cls, field_name: str) -> Any:
        """
        获取字段的缺省值
        """
        for f in fields(cls):   # type: ignore
            if f.name == field_name:
                return f.default if f.default is not MISSING else None
        raise ValueError(f"Field '{field_name}' not found in {cls.__name__}")

    @classmethod
    def get_metadata(cls, field_name: str, prop_name: str) -> Any:
        """
        获取字段的metadata属性
        """
        for f in fields(cls):  # type: ignore
            if f.name == field_name:
                return f.metadata.get(prop_name) if f.metadata is not MISSING else None
        raise ValueError(f"Field '{field_name}.{prop_name}' not found in {cls.__name__}")

    @property
    def to_map(self) -> dict[str, Any]:
        return {
            k: repr(v) for k, v in self.__dict__.items()
            if not (k.startswith('__') and k.endswith('__')) and k not in {'_text_format', '_data_formats'}
        }

    def set_random_value(self, field_name: str, max_value: Union[float,int] = None, min_value: Union[float,int] = 0) -> Any:
        """
        为指定字段随机生成值并设置到实例属性中。

        :param field_name: 数据类中的字段名
        :param max_value: 最大值（可选）
        :param min_value: 最小值（默认为0）
        """
        for f in fields(self):  # type: ignore
            # 处理 int 和 float 类型
            if f.name == field_name and issubclass(f.type, (int, float)):
                min_ = min_value if min_value else f.metadata.get('min', min_value) if f.metadata is not MISSING else min_value
                max_ = max_value if max_value else f.metadata.get('max', max_value) if f.metadata is not MISSING else max_value

                if min_ is None or max_ is None or min_ > max_:
                    raise ValueError("min_value 和 max_value 必须提供有效范围。")

                value = random.randint(min_, max_) if f.type == int else random.uniform(min_, max_)
                setattr(self, field_name, value)
                return value
        raise ValueError(f"Field '{field_name}' not found in {self.__name__}")

    def set_random_array(self, field_name: str, size: int, max_value: Union[float,int] = None, min_value: Union[float,int] = 0) -> Any:
        """
        为指定字段随机生成值并设置到实例属性中。

        :param field_name: 数据类中的字段名
        :param max_value: 最大值（可选）
        :param min_value: 最小值（默认为0）
        :param size: 数组的长度
        """
        for f in fields(self):  # type: ignore
            # 处理 list[int] 和 list[float] 类型
            if f.name == field_name and f.type in (list[int], list[float], list[int], list[float]):
                min_ = min_value if min_value else f.metadata.get('min', min_value) if f.metadata is not MISSING else min_value
                max_ = max_value if max_value else f.metadata.get('max', max_value) if f.metadata is not MISSING else max_value

                if min_ is None or max_ is None or min_ > max_:
                    raise ValueError("min_value 和 max_value 必须提供有效范围。")

                # 生成随机列表
                value = [
                    random.randint(min_, max_) if f.type == list[int] else random.uniform(min_, max_)
                    for _ in range(size)
                ]
                setattr(self, field_name, value)
                return value
        raise ValueError(f"Field '{field_name}' not found in {self.__name__}")

    def set_random_array_3d(
        self,
        field_name: str,
        shape: list[int],
        max_value: Union[int, float] = None,
        min_value: Union[int, float] = 0) -> Any:
        """
        为指定字段生成嵌套随机数组并设置到实例属性中。

        :param field_name: 数据类中的字段名
        :param shape: 数组的形状，例如 [2, 3, 4] 表示 2x3x4 的三维数组
        :param max_value: 最大值（可选）
        :param min_value: 最小值（默认为 0）
        """
        for f in fields(self):   # type: ignore
            if f.name == field_name and f.type in (list[list[list[int]]],
                                                   list[list[list[float]]],
                                                   list[list[list[Optional[int]]]],
                                                   list[list[list[Optional[float]]]],
                                                   list[list[list[int]]],
                                                   list[list[list[float]]],
                                                   list[list[list[Optional[int]]]],
                                                   list[list[list[Optional[float]]]]):
                min_ = min_value if min_value else f.metadata.get('min', min_value) if f.metadata is not MISSING else min_value
                max_ = max_value if max_value else f.metadata.get('max', max_value) if f.metadata is not MISSING else max_value

                if min_ is None or max_ is None or min_ > max_:
                    raise ValueError("min_value 和 max_value 必须提供有效范围。")

                # 递归生成嵌套数组
                def generate_nested_array(current_shape: list[int]) -> Any:
                    if len(current_shape) == 1:  # 最后一维生成随机值
                        return [
                            random.randint(min_, max_)
                            if f.type in [list[list[list[int]]], list[list[list[Optional[int]]]], list[list[list[int]]], list[list[list[Optional[int]]]]]
                            else random.uniform(min_, max_)
                            for _ in range(current_shape[0])
                        ]
                    return [
                        generate_nested_array(current_shape[1:])
                        for _ in range(current_shape[0])
                    ]

                # 生成随机嵌套数组
                value = generate_nested_array(shape)
                setattr(self, field_name, value)
                return value

        raise ValueError(f"Field '{field_name}' not found in {self.__class__.__name__}")



from dataclasses import dataclass, field
from typing import Optional

from mag_tools.bean.base_data import BaseData
from mag_tools.utils.data.list_utils import ListUtils
from mag_tools.utils.data.string_utils import StringUtils
from mag_tools.utils.data.value_utils import ValueUtils


@dataclass
class Dimension(BaseData):
    nx: Optional[int] = field(default=None, metadata={
        'min': 10,
        'max': 9999,
        'description': '行数'})
    ny: Optional[int] = field(default=None, metadata={
        'min': 10,
        'max': 9999,
        'description': '列数'})
    nz: Optional[int] = field(default=None, metadata={
        'min': 10,
        'max': 9999,
        'description': '层数'})
    ngrid: Optional[int] = field(default=None, metadata={'description': '总网格数'})

    @classmethod
    def from_block(cls, block_lines):
        if block_lines is None or len(block_lines) < 1:
            return None

        block_lines = [StringUtils.pick_head(line, '#') if '#' in line else line for line in block_lines]

        block_lines = ListUtils.trim(block_lines)

        return cls.__from_text(' '.join(block_lines))

    def to_block(self):
        if self.nx is not None and self.ny is not None and self.nz is not None and self.ngrid is not None:
            return []

        lines = ['DIMENS', self.__to_text()]
        return lines

    @property
    def size(self):
        if self.nx is not None and self.ny is not None and self.nz is not None:
            return self.nx * self.ny * self.nz
        else:
            return self.ngrid

    @property
    def shape(self) -> tuple[int, int, int]:
        return self.nx, self.ny, self.nz

    @classmethod
    def random_generate(cls):
        dimens = cls()
        dimens.set_random_value('nx', 10000)
        dimens.set_random_value('ny', 1000)
        dimens.set_random_value('nz', 100)
        return dimens

    @classmethod
    def __from_text(cls, text):
        if not text:
            return None

        numbers = ValueUtils.pick_numbers(text)
        nx = numbers[0] if len(numbers) == 3 else None
        ny = numbers[1] if len(numbers) == 3 else None
        nz = numbers[2] if len(numbers) == 3 else None
        ngrid = numbers[0] if len(numbers) == 1 else None
        return cls(nx=nx, ny=ny, nz=nz, ngrid=ngrid)

    def __to_text(self):
        if self.nx and self.ny and self.nz:
            return f' {self.nx} {self.ny} {self.nz}'
        elif self.ngrid:
            return f' {self.ngrid}'


from mag_tools.enums.base_enum import BaseEnum


class ServiceStatus(BaseEnum):
    # --- 1xx Informational ---
    CONTINUE = (100, "Continue", "继续")
    SWITCHING_PROTOCOLS = (101, "Switching Protocols", "交换协议")
    PROCESSING = (102, "Processing", "处理中")
    CHECKPOINT = (103, "Checkpoint", "检测点")
    # --- 2xx Success ---
    OK = (200, "OK", "正常")
    CREATED = (201, "Created", "已创建")
    ACCEPTED = (202, "Accepted", "已接受")
    NON_AUTHORITATIVE_INFORMATION = (203, "Non-Authoritative Information", "非授权信息")
    NO_CONTENT = (204, "No Content", "无内容")
    RESET_CONTENT = (205, "Reset Content", "重置内容")
    PARTIAL_CONTENT = (206, "Partial Content", "部分内容")
    MULTI_STATUS = (207, "Multi-Status", "多重状态")
    ALREADY_REPORTED = (208, "Already Reported", "已经报告")
    IM_USED = (226, "IM Used", "IM已使用")
    # --- 3xx Redirection ---
    MULTIPLE_CHOICES = (300, "Multiple Choices", "多重选择")
    MOVED_PERMANENTLY = (301, "Moved Permanently", "永久移除")
    FOUND = (302, "Found", "已发现")
    SEE_OTHER = (303, "See Other", "参见其它")
    NOT_MODIFIED = (304, "Not Modified", "未修改")
    TEMPORARY_REDIRECT = (307, "Temporary Redirect", "临时重定向")
    PERMANENT_REDIRECT = (308, "Permanent Redirect", "永久重定向")
    # --- 4xx Client Error ---
    BAD_REQUEST = (400, "Bad Request", "错误请求")
    UNAUTHORIZED = (401, "Unauthorized", "未经授权")
    PAYMENT_REQUIRED = (402, "Payment Required", "需要支付")
    FORBIDDEN = (403, "Forbidden", "被禁止")
    NOT_FOUND = (404, "Not Found", "未找到")
    METHOD_NOT_ALLOWED = (405, "Method Not Allowed", "方法不允许")
    NOT_ACCEPTABLE = (406, "Not Acceptable", "不可接受")
    PROXY_AUTHENTICATION_REQUIRED = (407, "Proxy Authentication Required", "需要代理身份验证")
    REQUEST_TIMEOUT = (408, "Request Timeout", "请求超时")
    CONFLICT = (409, "Conflict", "冲突")
    GONE = (410, "Gone", "已离开")
    LENGTH_REQUIRED = (411, "Length Required", "需要长度")
    PRECONDITION_FAILED = (412, "Precondition Failed", "先决条件失败")
    PAYLOAD_TOO_LARGE = (413, "Payload Too Large", "有效载荷太大")
    URI_TOO_LONG = (414, "URI Too Long", "URI太长")
    UNSUPPORTED_MEDIA_TYPE = (415, "Unsupported Media Type", "不支持的媒体类型")
    REQUESTED_RANGE_NOT_SATISFIABLE = (416, "Requested range not satisfiable", "请求范围不符合要求")
    EXPECTATION_FAILED = (417, "Expectation Failed", "预期失败")
    I_AM_A_TEAPOT = (418, "I'm a teapot", "“我是茶壶")
    UNPROCESSABLE_ENTITY = (422, "Unprocessable Entity", "不可处理实体")
    LOCKED = (423, "Locked", "已锁定")
    FAILED_DEPENDENCY = (424, "Failed Dependency", "依赖失败")
    TOO_EARLY = (425, "Too Early", "太早")
    UPGRADE_REQUIRED = (426, "Upgrade Required", "需要升级")
    PRECONDITION_REQUIRED = (428, "Precondition Required", "要求先决条件")
    TOO_MANY_REQUESTS = (429, "Too Many Requests", "请求太多")
    REQUEST_HEADER_FIELDS_TOO_LARGE = (431, "Request Header Fields Too Large", "请求头字段太大")
    UNAVAILABLE_FOR_LEGAL_REASONS = (451, "Unavailable For Legal Reasons", "因法律原因不可用")
    BUS_OPERATION_FAIL = (499, "业务处理失败", "业务处理失败")
    # --- 5xx Server Error ---
    INTERNAL_SERVER_ERROR = (500, "Internal Server Error", "内部服务器错误")
    BAD_GATEWAY = (502, "Bad Gateway", "网关故障")
    SERVICE_UNAVAILABLE = (503, "Service Unavailable", "服务不可用")
    GATEWAY_TIMEOUT = (504, "Gateway Timeout", "网关超时")
    HTTP_VERSION_NOT_SUPPORTED = (505, "HTTP Version not supported", "HTTP版本不支持")
    VARIANT_ALSO_NEGOTIATES = (506, "Variant Also Negotiates", "变体也在协商中")
    INSUFFICIENT_STORAGE = (507, "Insufficient Storage", "存储空间不足")
    LOOP_DETECTED = (508, "Loop Detected", "检测到循环")
    BANDWIDTH_LIMIT_EXCEEDED = (509, "Bandwidth Limit Exceeded", "超出带宽限制")
    NOT_EXTENDED = (510, "Not Extended", "未扩展")
    NETWORK_AUTHENTICATION_REQUIRED = (511, "Network Authentication Required", "需要网络身份验证")
    # --- 6xx DB Error ---
    DB_OPERATION_FAIL = (600, "数据操作失败", "数据操作失败")
    DUPLICATE_ENTRY = (601, "Duplicate entry", "数据重复")
    PARAMETER_MISS_MATCH = (602, "参数个数不匹配", "参数个数不匹配")
    CONNECT_FAIL = (603, "取数据库连接失败", "取数据库连接失败")
    INSERT_FAIL = (604, "插入失败", "插入失败")

    def __init__(self, code, reason, desc):
        super().__init__(code, desc)
        self.reason = reason

    @classmethod
    def of_reason(cls, reason):
        status = None
        if reason:
            for status in cls:
                if reason.startswith(status.reason):
                    break
        return status


import re
from datetime import date, datetime, time, timedelta
from typing import  Optional, Union

from dateutil import parser

from mag_tools.utils.data.value_utils import ValueUtils


class DateUtils:
    @staticmethod
    def to_string(dt: Union[datetime, date, time], pattern: str) -> str:
        return dt.strftime(pattern) if dt is not None else None

    """
    日期时间的工具类
    """
    @staticmethod
    def float_to_datetime(timestamp: float) -> datetime:
        """
        将浮点数转换为日期时间对象

        :param timestamp: 浮点数，表示自 Unix 纪元以来的秒数
        :return: 日期时间对象
        """
        return datetime(1970, 1, 1) + timedelta(seconds=timestamp)

    @staticmethod
    def to_datetime(date_time_str: str) -> Optional[datetime]:
        date_time_formats = [
            "%Y-%m-%d %H:%M:%S",
            "%Y/%m/%d %H:%M:%S",
            "%Y.%m.%d %H:%M:%S",
            "%Y年%m月%d日 %H时%M分%S秒",
            "%Y年%m月%d日%H时%M分%S秒",
            "%Y-%m-%d",
            "%Y/%m/%d",
            "%Y.%m.%d",
            "%H:%M:%S",
            "%Y年%m月%d日",
            "%H时%M分%S秒",
            "%Y-%m-%d %H:%M",
            "%Y-%m-%d %H:%M:%S.%f",
            "%Y%m%d%H%M%S",
            "%Y%m%d%H%M%S%f",
            "%Y%m%d",
            "%a, %d %b %Y %H:%M:%S %Z",
            "%a %b %d %H:%M:%S %Z %Y",
            "%Y-%m-%dT%H:%M:%S%z",
            "%Y-%m-%dT%H:%M:%S.%fZ",
            "%Y-%m-%dT%H:%M:%S%z",
            "%Y-%m-%dT%H:%M:%S.%f%z"
        ]

        for fmt in date_time_formats:
            try:
                return datetime.strptime(date_time_str, fmt)
            except ValueError:
                continue

        try:
            return parser.parse(date_time_str)
        except ValueError:
            return None

    @staticmethod
    def pick_datetimes(text: str) -> list[Union[datetime, date, time]]:
        """
        从字符串中提取日期、时间和日期时间。

        参数：
        :param text: 包含日期或时间的字符串
        :return: 包含日期、时间和日期时间的字典
        """
        datetime_pattern = r'(?:\d{4}[-/.年]\d{1,2}[-/.月]\d{1,2}(?:[日T\s])?)?(?:\d{1,2}[:时]\d{1,2}[:分]?(?:\d{1,2}(?:[秒])?)?)?'

        # 转换为对象
        results = []
        # 匹配日期时间
        for match in re.finditer(datetime_pattern, text):
            full_match = match.group(0)  # 获取完整匹配项
            dt = DateUtils.to_datetime(full_match)
            if dt is not None:
                if DateUtils.datetime_type(full_match) == datetime:
                    results.append(dt)
                elif DateUtils.datetime_type(full_match) == date:
                    results.append(dt.date())
                elif DateUtils.datetime_type(full_match) == time:
                    results.append(dt.time())
        return results

    @staticmethod
    def pick_datetime(text: str) -> Optional[datetime]:
        results = DateUtils.pick_datetimes(text)
        if not results:
            time_value = ValueUtils.to_value(text, float)
            if time_value is not None:
                return DateUtils.float_to_datetime(time_value)
        else:
            return results[0] if len(results) > 0 else None

    @staticmethod
    def datetime_type(text: str) -> Optional[type]:
        """
        从字符串中提取日期、时间和日期时间。

        参数：
        :param text: 包含日期或时间的字符串
        :return: 包含日期、时间和日期时间的字典
        """
        datetime_pattern = r'\d{4}[-/.年]\d{1,2}[-/.月]\d{1,2}(?:[日T\s])?[\s]?\d{1,2}[:时]\d{1,2}[:分](?:\d{1,2}[秒]?)?'
        date_pattern =     r'\d{4}[-/.年]\d{1,2}[-/.月]\d{1,2}(?:[日T\s])?'
        time_pattern = r'\d{1,2}[:时]\d{1,2}[:分]?(?:\d{1,2}(?:[秒])?)?'

        if re.match(datetime_pattern, text):
            return datetime
            # 判断日期
        elif re.match(date_pattern, text):
            return date
            # 判断时间
        elif re.match(time_pattern, text):
            return time
        else:
            return None


from dataclasses import dataclass, field
from datetime import datetime
from typing import Generic,  Optional, TypeVar, Union

from mag_tools.bean.db_list import DbList
from mag_tools.exception.app_exception import AppException
from mag_tools.enums.service_status import ServiceStatus

T = TypeVar('T')


@dataclass
class Results(Generic[T]):
    """
    服务器返回结果包
    """
    code: Optional[ServiceStatus] = field(default=None, metadata={'description': '返回代码'})
    message: Optional[str] = field(default=None, metadata={'description': '错误消息'})
    data: list[T] = field(default_factory=list, metadata={'description': '返回数据'})
    total_count: Optional[int] = field(default=None, metadata={'description': '返回数据的个数'})
    timestamp: Optional[datetime] = field(default_factory=datetime.now, metadata={'description': '时间戳'})

    @staticmethod
    def exception(ex: Exception):
        message = str(ex) if ex.args else str(ex.__cause__)
        return Results(code=ServiceStatus.INTERNAL_SERVER_ERROR, message=message)

    @staticmethod
    def success(data: Optional[Union[list[T],DbList[T]]] = None):
        if data is None:
            data = []

        total_count = len(data) if isinstance(data, list) else data.total_count

        return Results(message="OK", data=data if isinstance(data, list) else data.data, total_count=total_count)

    @staticmethod
    def fail(message: str):
        return Results(message=message)

    @property
    def is_success(self) -> bool:
        return self.code == ServiceStatus.OK.code

    @property
    def size(self) -> int:
        return len(self.data)

    @property
    def first(self) -> Optional[T]:
        return self.data[0] if self.data and len(self.data) > 0 else None

    def check(self) -> None:
        if not self.is_success:
            raise AppException(self.message)

    def get(self, idx: int) -> Optional[T]:
        self.check()
        return self.data[idx] if idx < self.size else None


from dataclasses import dataclass, field
from datetime import date, datetime, time
from typing import Any,  Optional

import numpy as np

from mag_tools.bean.array import Array
from mag_tools.enums.justify_type import JustifyType
from mag_tools.utils.data.value_utils import ValueUtils
from mag_tools.format.string_format import StringFormat


@dataclass
class TextFormatter:
    """
    文本拼接的格式，用于将多个字符串拼接为一个文本时使用
    """
    number_per_line: Optional[int] = field(default=None, metadata={"description": "每行显示的数据个数"})
    justify_type: Optional[JustifyType] = field(default=JustifyType.LEFT, metadata={"description": "对齐方式"})
    at_header: Optional[str] = field(default='', metadata={"description": "句首添加的字符串"})
    at_end: Optional[str] = field(default='', metadata={"description": "句尾添加的字符串"})
    decimal_places: Optional[int] = field(default=2, metadata={"description": "小数位数"})
    decimal_places_of_zero: Optional[int] = field(default=1, metadata={"description": "小数为0时的小数位数"})
    pad_length: Optional[int] = field(default=None, metadata={"description": "字段显示长度，为 None 保持字段原长度"})
    pad_char: Optional[str] = field(default=' ', metadata={"description": "填充占位符"})
    scientific: bool = field(default=False, metadata={"description": "是否使用科学计数法"})
    none_default: str = field(default=None, metadata={'description': 'int或float字段为None时的缺省值'})
    group_by_layer: bool = field(default=True, metadata={'description': "对3D数组分层展示，对2D数组分行展示"})
    merge_duplicate: bool = field(default=False, metadata={'description': '是否合并重复的数据'})

    def array_1d_to_lines(self, array_1d: list[Any]) -> list[str]:
        """
        将一维数组转换为多行文本表示,相邻的重复数据采用 m*A的方法
        :param array_1d: 一维数组，包含任意类型的数值
        :return: 多行文本表示的数组
        """
        result = self.__merge_array_1d(array_1d)
        max_length = len(max((s for s in result if isinstance(s, str)), key=len, default=""))
        return self.__to_lines(result, max_length)

    def array_2d_to_lines(self, array_2d: list[list[Any]]) -> list[str]:
        """
        将二维数组平铺为一维，然后生成数值块,相邻的重复数据采用 m*A的方法
        :param array_2d: 二维数组
        :return: 数值块，由多行构成，每行由 "数值" 或 "数值个数*数值"构成
        """
        if self.group_by_layer:
            new_array_2d = []
            # 遍历每一行后添加到列表中
            num_rows = np.array(array_2d).shape[0]  # 行数
            for i in range(num_rows):
                new_array_2d.append(self.__merge_array_1d(array_2d[i]))

            array_1d_sum = self.__to_blocks(new_array_2d)
        else:
            # 将其展平为一维数组后添加到列表中
            layer = np.array(array_2d).flatten()
            array_1d_sum = self.array_1d_to_lines(layer.tolist())
        return array_1d_sum

    def array_3d_to_lines(self, array_3d: list[list[list[Any]]]) -> list[str]:
        """
        将三维数组按层平铺为二维，然后生成数值块,相邻的重复数据采用 m*A的方法
        :param array_3d: 三维数组
        :return: 数值块，由多行构成，每行由 "数值" 或 "数值个数*数值"构成
        """
        #分层展示
        if self.group_by_layer:
            new_array_2d = []
            # 遍历每一层并将其展平为一维数组后添加到列表中
            num_layers = np.array(array_3d).shape[0]  # 层数
            for i in range(num_layers):
                array_1d = np.array(array_3d[i]).flatten().tolist()
                new_array_2d.append(self.__merge_array_1d(array_1d))

            array_1d_sum = self.__to_blocks(new_array_2d)
        #所有数据合并一个块后展示
        else:
            array_1d = np.array(array_3d).flatten()
            array_1d_sum = self.array_1d_to_lines(array_1d.tolist())

        return array_1d_sum

    def arrays_to_lines(self, arrays: list[Array]) -> list[str]:
        self.pad_length = 0
        blocks = []
        for array_ in arrays:
            items = [array_.array_name, array_.unit_name]
            items.extend(array_.data)
            blocks.append(items)

        return self.__to_blocks(blocks)

    def __merge_array_1d(self, array_1d: list[Any]) -> list[Any]:
        """
        合并一维数组,相邻的重复数据采用 m*A的方法
        :param array_1d: 一维数组，包含任意类型的数值
        """
        if array_1d is None:
            return []

        result = []

        #需要合并重复数据
        if self.merge_duplicate:
            count = 1
            current_value = array_1d[0]

            for i in range(1, len(array_1d)):
                if array_1d[i] == current_value:
                    count += 1
                else:
                    current_text = ValueUtils.to_string(current_value, self.decimal_places,
                                                        self.decimal_places_of_zero, self.scientific)
                    if current_text is None:
                        current_text = self.none_default

                    txt = f"{count}*{current_text}" if count > 1 else f"{current_text}"
                    result.append(txt)

                    current_value = array_1d[i]
                    count = 1

            # 处理最后一组数据
            current_text = ValueUtils.to_string(current_value, self.decimal_places,
                                                self.decimal_places_of_zero, self.scientific)
            if current_text is None:
                current_text = self.none_default
            txt = f"{count}*{current_text}" if count > 1 else f"{current_text}"
            result.append(txt)
        else:
            for v in array_1d:
                result.append(ValueUtils.to_string(v, self.decimal_places, self.decimal_places_of_zero, self.scientific, self.none_default))

        return result

    def __to_lines(self, values: list[Any], max_len: int) -> list[str]:
        """
        将数值数组转为按指定格式的字符串
        :param values: 数值数组
        :param max_len: 字段显示的最大长度,为None时则按格式中定义的长度
        """
        padding_length = max(self.pad_length, max_len) if self.pad_length is not None else None

        lines = list()
        for value in values:
            if isinstance(value, int) or isinstance(value, float):
                value_str = ValueUtils.to_string(value=value,
                                                 decimal_places=self.decimal_places,
                                                 decimal_places_of_zero=self.decimal_places_of_zero,
                                                 scientific=self.scientific,
                                                 none_default=self.none_default)
            elif isinstance(value, date):
                value_str = value.strftime('%Y-%m-%d')
            elif isinstance(value, time):
                value_str = value.strftime('%H:%M:%S')
            elif isinstance(value, datetime):
                value_str = value.strftime('%Y-%m-%d %H:%M:%S')
            else:
                value_str = str(value)

            lines.append(value_str if value_str is not None else self.none_default)

        return StringFormat.pad_text(strings=lines,
                                     padding_length=padding_length,
                                     justify=self.justify_type,
                                     number_per_line=self.number_per_line,
                                     at_header=self.at_header,
                                     at_end=self.at_end,
                                     pad_char=self.pad_char)

    def __to_blocks(self, values: list[list[Any]]) -> list[str]:
        max_len = max(len(str(item)) for sublist in values for item in sublist)

        lines = []
        for value in values:
            lines.extend(self.__to_lines(value, max_len))
        return lines


import hashlib
import os
import platform
import shutil
import string
from pathlib import Path

from mag_tools.enums.digest_alg import DigestAlg


class FileUtils:
    @staticmethod
    def is_system_path(path: str) -> bool:
        """
        判断一个目录是否为系统目录
        :param path: 目录路径
        :return: 如果是系统目录返回 True，否则返回 False
        """
        system_directories = []

        if platform.system() == "Windows":
            user_profile = os.environ.get("USERPROFILE", "C:\\Users\\Default").lower()

            # 获取所有驱动器的根目录
            drives = [f"{d}:" for d in string.ascii_lowercase if os.path.exists(f"{d}:")]

            system_directories = drives + [
                os.path.normpath(os.environ.get("SystemDrive", "C:")).lower(),
                os.path.normpath(os.environ.get("HOMEDRIVE", "C:")).lower(),
                os.path.normpath(os.environ.get("SystemRoot", "C:\\Windows")).lower(),
                os.path.normpath(os.environ.get("ProgramFiles", "C:\\Program Files")).lower(),
                os.path.normpath(os.environ.get("ProgramFiles(x86)", "C:\\Program Files (x86)")).lower(),
                os.path.normpath(os.environ.get("Public", "C:\\Users\\Public")).lower(),
                os.path.normpath(os.environ.get("USERPROFILE", "C:\\Users\\Default")).lower(),
                os.path.normpath(os.environ.get("HOMEPATH", "C:\\Users\\Default")).lower(),
                os.path.join(user_profile, "Desktop").lower(),
                os.path.join(user_profile, "Documents").lower(),
                os.path.join(user_profile, "Downloads").lower(),
                os.path.join(user_profile, "Pictures").lower(),
                os.path.normpath("C:\\Windows\\System32").lower(),
                os.path.normpath("C:\\Windows\\SysWOW64").lower(),
                os.path.normpath("C:\\Users\\Default").lower(),
                os.path.normpath("C:\\ProgramData").lower(),
                os.path.normpath("C:\\Users").lower()
            ]
        elif platform.system() == "Linux":
            system_directories = [
                "/",
                "/bin", "/sbin", "/usr", "/lib", "/etc",
                "/boot", "/dev", "/home", "/opt", "/root", "/srv", "/var"
            ]

        # 标准化输入路径并去掉末尾的反斜杠
        normalized_path = os.path.normpath(path).lower().rstrip("\\")

        # 检查目录路径是否在系统目录列表中
        if normalized_path in system_directories:
            return True

        return False

    @staticmethod
    def copy_and_overwrite(src: str, dst: str):
        """
        将源文件覆盖目标文件
        :param src: 源文件路径
        :param dst: 目标文件路径
        """
        src_path = Path(src)
        dst_path = Path(dst)

        # 确保目标文件所在目录存在
        dst_path.parent.mkdir(parents=True, exist_ok=True)

        # 复制并覆盖文件
        shutil.copy2(src_path, dst_path)

    @staticmethod
    def clear_dir(dir_name):
        if os.path.exists(dir_name):
            if FileUtils.is_system_path(dir_name):
                raise ValueError(f"不能清除系统目录[{dir_name}]下的文件")

            for filename in os.listdir(dir_name):
                file_path = os.path.join(dir_name, filename)  # 构建文件的完整路径
                try:
                    if os.path.isfile(file_path):
                        os.remove(file_path)
                    else:
                        shutil.rmtree(file_path)
                except Exception as e:
                    raise ValueError(f"Error removing {file_path}: {e}")

    @staticmethod
    def delete_dir(dir_name):
        if os.path.exists(dir_name):
            if FileUtils.is_system_path(dir_name):
                raise ValueError(f"不能清除系统目录[{dir_name}]下的文件")

            shutil.rmtree(dir_name)

    @staticmethod
    def delete_file(file_path):
        if os.path.exists(file_path):
            file_dir = os.path.dirname(file_path)
            if FileUtils.is_system_path(file_dir):
                raise ValueError(f"不能清除系统目录[{file_dir}]下的文件")

            if os.path.isfile(file_path):
                os.remove(file_path)

    @staticmethod
    def file_size(file_path):
        if os.path.isfile(file_path):
            return os.path.getsize(file_path)
        else:
            return None

    @staticmethod
    def file_hash(file_path, algorithm=DigestAlg.MD5):
        """
        计算文件内容的哈希值

        :param file_path: 文件路径
        :param algorithm: 哈希算法（默认使用 'sha256'）
        :return: 文件哈希值的十六进制字符串表示
        """
        hash_func = hashlib.new(algorithm.code)
        with open(file_path, 'rb') as f:
            while chunk := f.read(8192):
                hash_func.update(chunk)
        return hash_func.hexdigest()


if __name__ == '__main__':
    # 示例调用
    _file = "D:\\HiSimPack\\data\\Comp\\spe9\\SPE9.dat"
    hash_ = FileUtils.file_hash(_file)
    print(hash_)



from dataclasses import dataclass, field
from typing import Optional

import wmi

from mag_tools.bean.base_data import BaseData
from mag_tools.enums.media_type import MediaType
from mag_tools.utils.security.digest import Digest


@dataclass
class Disk(BaseData):
    """
    磁盘参数类
    """
    computer_id: Optional[str] = field(default=None, metadata={"description": "所属计算机的标识"})
    serial_number: Optional[str] = field(default=None, metadata={"description": "序列号"})
    model: Optional[str] = field(default=None, metadata={"description": "型号"})
    media_type: Optional[str] = field(default=None, metadata={"description": "介质类型"})
    manufacturer: Optional[str] = field(default=None, metadata={"description": "制造商"})
    capacity: Optional[int] = field(default=None, metadata={"description": "总容量，单位为G"})
    caption: Optional[str] = field(default=None, metadata={"description": "描述"})
    device_id: Optional[str] = field(default=None, metadata={"description": "磁盘的设备标识符"})
    partitions: Optional[int] = field(default=None, metadata={"description": "分区数"})
    interface_type: Optional[str] = field(default=None, metadata={"description": "接口类型"})
    firmware_revision: Optional[str] = field(default=None, metadata={"description": "固件版本"})
    status: Optional[str] = field(default=None, metadata={"description": "状态"})
    sectors_per_track: Optional[int] = field(default=None, metadata={"description": "每磁道的扇区数"})
    total_tracks: Optional[int] = field(default=None, metadata={"description": "磁盘总磁道数"})
    bytes_per_sector: Optional[int] = field(default=None, metadata={"description": "每扇区的字节数"})
    tracks_per_cylinder: Optional[int] = field(default=None, metadata={"description": "每柱面的磁道数"})

    @property
    def hash(self):
        return Digest.md5(f'{self.serial_number}{self.media_type}{self.model}{self.manufacturer}{self.capacity}')

    @classmethod
    def get_info(cls):
        physical_disks = []
        c = wmi.WMI()
        for disk in c.Win32_DiskDrive():
            info = cls(serial_number=disk.DeviceID,
                       media_type=MediaType.of_code(disk.MediaType),
                       model=disk.Model,
                       capacity=int(disk.Size) // 1000**3,
                       manufacturer=disk.Manufacturer,
                       caption=disk.Caption,
                       device_id=disk.DeviceID,
                       partitions=disk.Partitions,
                       interface_type=disk.InterfaceType,
                       firmware_revision=disk.FirmwareRevision,
                       status=disk.Status,
                       sectors_per_track=disk.SectorsPerTrack,
                       total_tracks=disk.TotalTracks,
                       bytes_per_sector=disk.BytesPerSector,
                       tracks_per_cylinder=disk.TracksPerCylinder)

            if "SSD" in disk.Model or "Solid State" in disk.Model:
                info.media_type = MediaType.SSD

            physical_disks.append(info)

        return physical_disks

    @classmethod
    def __parse(cls, model):
        items = model.split()
        manufacturer = items[0] if len(items) > 0 else None
        dick_type = MediaType.of_code(items[1]) if len(items) > 1 else None

        return manufacturer, dick_type

if __name__ == '__main__':
    disk_ = Disk.get_info()
    print(disk_)


import json
import os

from mag_tools.jsonparser.json_encoder import JsonEncoder


class JsonFileUtils:
    @staticmethod
    def update_params(file_path: str, new_params: dict[str, str]):
        """
            从 JSON 文件中读取参数，替代其中部分参数，并保存回文件
            :param file_path: JSON 文件路径
            :param new_params: 需要更新的参数字典 """

        with open(file_path, 'r+', encoding='utf-8') as f:
            data = json.load(f)
            data.update(new_params)

        new_data_str = json.dumps(data, ensure_ascii=False, indent=4, cls=JsonEncoder)
        with open(file_path, 'w', encoding='utf-8') as f:
            f.write(new_data_str)

    @staticmethod
    def save_as_json(json_file, data):
        """
        将字典保存为 JSON 文件
        :param data: 数据对象
        :param json_file: JSON文件路径
        """
        filtered_data = JsonFileUtils.__remove_none(data)
        data_str = json.dumps(filtered_data, ensure_ascii=False, indent=4, cls=JsonEncoder)

        # 如果目录不存在，则创建目录
        os.makedirs(os.path.dirname(json_file), exist_ok=True)

        with open(json_file, 'w', encoding='utf-8') as jf:
            jf.write(data_str)

    @staticmethod
    def load_json(json_file) -> dict:
        with open(json_file, encoding='utf-8') as jf:
            data = json.load(jf)
            return data

    @staticmethod
    def load_json_string(json_file) -> str:
        with open(json_file, encoding='utf-8') as jf:
            data = json.load(jf)
            json_str = json.dumps(data, ensure_ascii=False, indent=4, cls=JsonEncoder)
        return json_str

    # 递归过滤掉值为 None 的键
    @staticmethod
    def __remove_none(d):
        if isinstance(d, dict):
            return {k: JsonFileUtils.__remove_none(v) for k, v in d.items() if v is not None}
        elif isinstance(d, list):
            return [JsonFileUtils.__remove_none(i) for i in d if i is not None]
        else:
            return d



import unittest
from datetime import datetime, date, time
from dateutil.tz import tzoffset

from mag_tools.utils.data.date_utils import DateUtils


class TestDateUtils(unittest.TestCase):
    def test_float_to_datetime(self):
        # 测试 Unix 时间戳转换
        self.assertEqual(
            DateUtils.float_to_datetime(0),
            datetime(1970, 1, 1)
        )
        self.assertEqual(
            DateUtils.float_to_datetime(86400),
            datetime(1970, 1, 2)
        )

    def test_to_datetime(self):
        # 测试不同格式的日期时间解析
        self.assertEqual(
            DateUtils.to_datetime("2025-03-10 14:30:00"),
            datetime(2025, 3, 10, 14, 30)
        )
        self.assertEqual(
            DateUtils.to_datetime("2025/03/10"),
            datetime(2025, 3, 10)
        )
        self.assertEqual(
            DateUtils.to_datetime("2025-03-10T14:30:00+0800"),
            datetime(2025, 3, 10, 14, 30, tzinfo=tzoffset(None, 28800))
        )
        self.assertIsNone(
            DateUtils.to_datetime("invalid format")
        )

    def test_pick_datetime(self):
        # 测试从文本中提取日期、时间和日期时间
        text = "会议时间是2025-03-10 16:30:45，截止日期是2025/03/15，#2025-05-05 05:05:05提醒设置在15:45。"
        expected_results = [
            datetime(2025, 3, 10, 16, 30, 45),
            date(2025, 3, 15),
            datetime(2025, 5, 5, 5, 5, 5),
            time(15, 45),
        ]
        self.assertEqual(DateUtils.pick_datetimes(text), expected_results)

        text = "这是一个空字符串，没有日期或时间。"
        self.assertEqual(DateUtils.pick_datetimes(text), [])

    def test_datetime_type(self):
        str_1 = '2025-03-10 16:30:45'
        str_2 = '2025-03-10'
        str_3 = '16:30:45'
        str_4 = '2025/3/10 16:30:45'
        str_5 = '2025/03/10'
        str_6 = '16:30'
        str_7 = '2025年03月10日 16时30分45秒'
        str_8 = '2025年03月10日'
        str_9 = '16时30分45秒'

        self.assertEqual(DateUtils.datetime_type(str_1), datetime)
        self.assertEqual(DateUtils.datetime_type(str_2), date)
        self.assertEqual(DateUtils.datetime_type(str_3), time)
        self.assertEqual(DateUtils.datetime_type(str_4), datetime)
        self.assertEqual(DateUtils.datetime_type(str_5), date)
        self.assertEqual(DateUtils.datetime_type(str_6), time)
        self.assertEqual(DateUtils.datetime_type(str_7), datetime)
        self.assertEqual(DateUtils.datetime_type(str_8), date)
        self.assertEqual(DateUtils.datetime_type(str_9), time)

if __name__ == "__main__":
    unittest.main()



from dataclasses import dataclass, field
from typing import Optional

import psutil

from mag_tools.bean.base_data import BaseData
from mag_tools.enums.fs_type import FsType

@dataclass
class DiskPartition(BaseData):
    computer_id: Optional[str] = field(default=None, metadata={"description": "所属计算机的标识"})
    device: Optional[str] = field(default=None, metadata={"description": "设备"})
    fs_type: Optional[FsType] = field(default=None, metadata={"description": "文件系统"})
    mount_point: Optional[str] = field(default=None, metadata={"description": "驱动装载点"})
    opts: Optional[str] = field(default=None, metadata={"description": "挂载选项"})

    @classmethod
    def get_info(cls):
        partition_infos = []

        partitions = psutil.disk_partitions()
        for partition in partitions:
            info = cls(device=partition.device,
                       fs_type=FsType.of_code(partition.fstype),
                       mount_point=partition.mountpoint,
                       opts=partition.opts)
            partition_infos.append(info)

        return partition_infos

if __name__ == '__main__':
    info_ = DiskPartition.get_info()
    print(info_)


import unittest
from dataclasses import dataclass
from datetime import datetime, date
from decimal import Decimal
from typing import Optional

from mag_tools.bean.easy_map import EasyMap
from mag_tools.enums.base_enum import BaseEnum


class TestEasyMap(unittest.TestCase):
    def setUp(self):
        self.map = EasyMap()

    def test_add_and_get_string(self):
        self.map.add("key1", "value1")
        self.assertEqual(self.map.get_string("key1"), "value1")

    def test_get_integer(self):
        self.map.add("key2", "123")
        self.assertEqual(self.map.get_int("key2"), 123)

    def test_get_double(self):
        self.map.add("key3", "123.45")
        self.assertEqual(self.map.get_float("key3"), 123.45)

    def test_get_boolean(self):
        self.map.add("key4", "true")
        self.assertTrue(self.map.get_bool("key4"))

    def test_get_date(self):
        self.map.add("key5", "2024-12-31")
        self.assertEqual(self.map.get_date("key5"), date(2024, 12, 31))

    def test_get_local_date_time(self):
        self.map.add("key6", "2024-12-31 23:59:59")
        self.assertEqual(self.map.get_datetime("key6"), datetime(2024, 12, 31, 23, 59, 59))

    def test_get_big_decimal(self):
        self.map.add("key7", "12345.6789")
        self.assertEqual(self.map.get_decimal("key7"), Decimal("12345.6789"))

    def test_get_enum(self):
        class Color(BaseEnum):
            RED = "red"
            GREEN = "green"
            BLUE = "blue"

        self.map.add("key8", "RED")
        self.assertEqual(self.map.get_enum("key8", Color), Color.RED)

    def test_to_map(self):
        self.map.add("key1", "value1")
        self.map.add("key2", "value2")
        expected_map = {"key1": "value1", "key2": "value2"}
        self.assertEqual(self.map.to_map(), expected_map)

    def test_get_by_json(self):
        self.map.add("key9", '{"name": "John", "age": 30}')
        result = self.map.get_by_json("key9", dict)
        self.assertEqual(result, {"name": "John", "age": 30})

    def test_keys_to_hump(self):
        @dataclass
        class ChildInfo:
            first_name: Optional[str] = None
            real_age: Optional[int] = None
            real_height: Optional[int] = None

        map_ = EasyMap().add("key_a1", "value1").add("key_a2", "value2").add("child", ChildInfo(first_name="John", real_age=30, real_height=100))
        new_map = map_.keys_to_hump()
        print(new_map)

if __name__ == "__main__":
    unittest.main()





from mag_tools.enums.base_enum import BaseEnum


class MediaType(BaseEnum):
    """
    磁盘类型枚举类
    """
    SSD = ("Solid state disk", "固态硬盘")
    HDD = ("Fixed hard disk media", "机械硬盘")
    REMOVABLE = ("Removable media", "可移动硬盘和U盘")
    OPTICAL = ("Optical disk", "光盘")
    UNKNOWN = ("Unknown", "未知")



from typing import  Iterator, Optional, TypeVar, Generic

from mag_tools.bean.db_page import DbPage

T = TypeVar('T')

class DbList(Generic[T]):
    def __init__(self, data: list[T] = None, page: Optional[DbPage] = None, total_count: int = -1):
        self.__data = data if data is not None else []
        self.__page = page if page is not None else DbPage(1, 10)
        self.total_count = total_count

    def set_page(self, page: DbPage):
        self.__page = page if page is not None else DbPage(1, 10)

    @property
    def size(self) -> int:
        return len(self.__data)

    def add(self, e: T):
        self.__data.append(e)

    def get(self, idx: int) -> T:
        if idx < 0 or idx >= self.size:
            return None
        return self.__data[idx]

    @property
    def is_empty(self) -> bool:
        return self.size == 0

    def clear(self):
        self.__data.clear()
        self.total_count = 0
        self.__page = None

    @property
    def data(self) -> list[T]:
        if self.__page:
            start = (self.__page.index - 1) * self.__page.size
            end = start + self.__page.size
            return self.__data[start:end]
        return self.__data

    def __iter__(self) -> Iterator[T]:
        return iter(self.__data)

    def __getitem__(self, idx: int) -> T:
        return self.get(idx)

    def __setitem__(self, idx: int, value: T):
        if 0 <= idx < self.size:
            self.__data[idx] = value
        else:
            raise IndexError("Index out of range")

    def __delitem__(self, idx: int):
        if 0 <= idx < self.size:
            del self.__data[idx]
        else:
            raise IndexError("Index out of range")



import unittest

from jsonparser.json_parser import JsonParser
from mag_tools.bean.db_list import DbList
from mag_tools.bean.results import Results


class TestResults(unittest.TestCase):
    def test(self):
        d = [1,2,3]
        data = DbList(d, total_count=10)
        results = Results.success(data)
        print(JsonParser.from_results(results))


from enum import Enum
from typing import Type


class EnumUtils:
    @staticmethod
    def get_enum_type(enum_cls: Type[Enum], variable_name: str):
        for member in enum_cls:
            if hasattr(member, variable_name):
                return type(getattr(member, variable_name))
        return None